/***********************************************************************
 * Macro qui assigne les deux dimensions de l'indice de          *******
 * dfavorisation qubcois et des indices locaux (RSS, RLS et   *******
 * CLCS), la zone gographique et le code ZIM de Statistique     *******
 * Canada  partir du code postal et du code municipal  tout    *******
 * fichier administratif comprenant au moins le code postal. On  *******
 * utilise une version simplifie du FCCP+ produit par   	     *******
 * Statistique Canada. En tout, le programme fait appel         *******
 * trois fichiers de conversion.              					 *******
 *  - WCF du FCCP+                                               *******
 *  - PCCFDUPS du FCCP+                                          *******
 *  - PCCFUNIQ du FCCP+                                          *******
 * Paramtres de la macro                                        *******
 *  - IN: fichier SAS d'entre auquel on dsire ajouter la       *******
 *        variable AD (aire de diffusion) et le cas chant      *******
 *        l'indice de dfavorisation (materiel et social)        *******
 *  - PCODE: nom de la variable du code postal  6 positions     *******
 *  - MUNIC: nom de la variable du code municipal. Par dfaut,   *******
 *           0 = l'attribution ne se fait qu' partir du code    *******
 *           postal seulement                                    *******
 *  - OUT: nom du fichier SAS de sortie identique  IN mais avec *******
 *         les nouvelles variables AD et indices de              *******
 *         dfavorisation                                        *******
 * Auteur: Denis Hamel, INSPQ                                    *******
 *         Philippe Gamache, INSPQ                               *******
 * Dernire modification: 22 avril 2020                           ******
 * Modifications: Ajout des quintiles rgionaux			          ******
 *                Remplacement du fichier munic			          ******
 ***********************************************************************/


*** remplacer les chemins d'accs aux fichiers selon vos emplacements ***;
libname i2001 "E:\Projets\IndiceDfavorisation\Indice2001\Assignation\Qubec";

filename fccp "E:\Projets\IndiceDfavorisation\Indice2001\Assignation\Qubec\fccp.txt";
filename munic "E:\Projets\IndiceDfavorisation\Indice2001\Assignation\Qubec\munic2001.txt";
filename dec04 "E:\Projets\IndiceDfavorisation\Indice2001\Assignation\Qubec\dec04.txt"; /*exemple de fichier texte contenant des codes postaux */


*** lire un fichier texte avec SAS ***;
data i2001.dec04;
infile i2001.dec04;
input  cp $ 1-6 cod_mun 7;
run;

data i2001.wcf;
infile fcp;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29 quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42 zone 44 zim 45 rss 46-47 nombre 48 clsc $ 49-53 RLS_code 54-57 quartmatclsc 58
       quartsocCLSC 59 quartmatRLS 60 quartsocRLS 61 quartmatRSS 62 quartsocRSS 63 frac 64-66 frac2 67-69 propmunic_CLSC 70-72 propcp_CLSC 73-75 
       propmunic_RLS 76-78 propcp_RLS 79-81 first 82 quintmatRSS 83 quintsocRSS 84;
run;

data i2001.pccfuniq;
infile pccfuniq;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29 quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42 zone 44 zim 45 rss 46-47 nombre 48 clsc $ 49-53 RLS_code 54-57 quartmatclsc 58
       quartsocCLSC 59 quartmatRLS 60 quartsocRLS 61 quartmatRSS 62 quartsocRSS 63 frac 64-66 frac2 67-69 propmunic_CLSC 70-72 propcp_CLSC 73-75 
       propmunic_RLS 76-78 propcp_RLS 79-81 quintmatRSS 82 quintsocRSS 83;
run;

data i2001.pccfdups;
infile pccfdups;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29 quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42 zone 44 zim 45 rss 46-47 nombre 48 clsc $ 49-53 RLS_code 54-57 quartmatclsc 58
       quartsocCLSC 59 quartmatRLS 60 quartsocRLS 61 quartmatRSS 62 quartsocRSS 63 frac 64-66 frac2 67-69 propmunic_CLSC 70-72 propcp_CLSC 73-75 
       propmunic_RLS 76-78 propcp_RLS 79-81 first 82 quintmatRSS 83 quintsocRSS 84;
run;

data i2001.munic;
  infile munic;
  input munic 1-7 mun2011 8-14;
run;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;

/**** Ne pas changer la macro qui suit *****/
%macro Assignation01(in,pcode,munic,out);

data wcf;
   set i2001.wcf;
   nuniq = _n_;
run;

data pccfuniq;
   set i2001.pccfuniq;
   nuniq = _n_;
run;

data pccfdups;
   set i2001.pccfdups;
   nuniq = _n_;
run;

proc sort data=i2001.munic;by munic;run;
proc sort data=wcf;by munic;run;
proc sort data=pccfuniq;by munic;run;
proc sort data=pccfdups;by munic;run;

data wcf;
  merge i2001.munic(in=in2) wcf(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

data pccfuniq;
  merge i2001.munic(in=in2) pccfuniq(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

data pccfdups;
  merge i2001.munic(in=in2) pccfdups(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

data fichin;
  %if &munic ne 0 %then %do; length munic 8.; %end;
  set &in;
  numero=_n_;
  nba=ranuni(766);
  pcode=compress(&pcode);
  %if &munic ne 0 %then %do; munic=2400000+mod(&munic,2400000); %end;
run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
  data fichin;
  set fichin;
  where zone in (.,0);
  run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;

%if &munic ne 0 %then %do;

proc sort data=i2001.munic;by munic;run;
proc sort data=fichin;by munic;run;


data fichin;
  merge i2001.munic(in=in2) fichin(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;


/*********************************************************
  * Assignation des indices de dfavorisation  partir ***
  * du code postal et du code de la municipalit       ***
  * Fichier FCCP+                                      ***
  *******************************************************/

/*** Avec WCF *****/
/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.propcp_CLSC, wcf.propmunic_CLSC, wcf.diffmatmunic, 
         wcf.diffsocmunic, wcf.fortcp, wcf.fortmunic, wcf.zim, wcf.zone, wcf.rss, wcf.centmat, wcf.nuniq,
         wcf.nombre, wcf.clsc, wcf.rls_code, wcf.frac, wcf.quartmatclsc, wcf.quartsocclsc
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq; 
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.propcp_CLSC, wcf.propmunic_RLS, wcf.diffmatmunic,
         wcf.diffsocmunic, wcf.fortcp, wcf.fortmunic, wcf.zim, wcf.zone, wcf.rss, wcf.centmat,wcf.nuniq,
         wcf.nombre, wcf.clsc, wcf.rls_code, wcf.frac, wcf.quartmatrls, wcf.quartsocrls
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq; 
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

  /*** RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, numero, nba, wcf.first, wcf.propmunic, wcf.diffmatmunic, wcf.diffsocmunic, wcf.fortcp, wcf.nuniq,
         wcf.fortmunic, wcf.quintmat, wcf.quintsoc, wcf.centmat, wcf.centsoc, wcf.zim, wcf.zone, wcf.rss, wcf.nombre, wcf.clsc, 
         wcf.quartmatRSS, wcf.quartsocRSS, wcf.quintmatRSS, wcf.quintsocRSS
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=fichin;by numero;run; 

data appmunic1 nonapp1;
  merge fichin(in=in1) test1(in=in2) test2(in=in2a) test3(in=in2b);
  by numero;
  if in1*in2*in2a*in2b then output appmunic1;
  else if in1 then output nonapp1;
run;

 /*** Avec PCCFUNIQ *****/
 /*** CLSC ***/
proc sort data=nonapp1;by pcode munic;run;
proc sort data=pccfuniq;by pcode munic;run;

proc sql;
  create table test as
  select nonapp1.pcode, nonapp1.munic, nonapp1.numero, nonapp1.nba, pccfuniq.propcp_CLSC, pccfuniq.propmunic_CLSC, pccfuniq.zim,
		 pccfuniq.zone, pccfuniq.centmat,pccfuniq.nuniq, pccfuniq.rss, pccfuniq.nombre, pccfuniq.clsc, pccfuniq.rls_code, pccfuniq.frac,
         pccfuniq.quartmatclsc, pccfuniq.quartsocclsc
  from nonapp1, pccfuniq
  where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par CLSC ***/

data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RLS ***/
proc sort data=nonapp1;by pcode munic;run;
proc sort data=pccfuniq;by pcode munic;run;

proc sql;
  create table test as
  select nonapp1.pcode, nonapp1.munic, nonapp1.numero, nonapp1.nba, pccfuniq.propcp_RLS, pccfuniq.propmunic_RLS,
		 pccfuniq.zim, pccfuniq.zone, pccfuniq.centmat,pccfuniq.nuniq, pccfuniq.rss, pccfuniq.nombre, pccfuniq.clsc, 
         pccfuniq.rls_code, pccfuniq.frac, pccfuniq.quartmatrls, pccfuniq.quartsocrls
  from nonapp1, pccfuniq
  where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par RLS ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RSS ***/
proc sql;
  create table test as
  select nonapp1.pcode, nonapp1.munic, numero, nba, pccfuniq.propmunic, pccfuniq.diffmatmunic, pccfuniq.diffsocmunic, pccfuniq.fortcp,
         pccfuniq.fortmunic, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc, pccfuniq.zim, pccfuniq.zone,
         pccfuniq.nuniq, pccfuniq.rss, pccfuniq.nombre, pccfuniq.clsc, pccfuniq.rls_code, pccfuniq.frac,
         pccfuniq.quartmatrss, pccfuniq.quartsocrss, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS
  from nonapp1, pccfuniq
  where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp1;by numero;run; 

data appmunic2 nonapp2;
  merge nonapp1(in=in1) test1(in=in2) test2(in=in2a) test3(in=in2b);
  by numero;
  if in1*in2*in2a*in2b then output appmunic2;
  else if in1 then output nonapp2;
run;

/*** Avec PCCFDUPS *****/
/*** CLSC ***/
proc sql;
  create table test as
  select nonapp2.pcode, nonapp2.munic, numero, nba, pccfdups.propmunic_CLSC, pccfdups.diffmatmunic, pccfdups.diffsocmunic, pccfdups.fortcp,
         pccfdups.nuniq, pccfdups.fortmunic, pccfdups.zim, pccfdups.zone,pccfdups.centmat, pccfdups.rss, pccfdups.nombre, pccfdups.clsc, 
         pccfdups.rls_code, pccfdups.frac, pccfdups.quartmatclsc, pccfdups.quartsocclsc
  from nonapp2, pccfdups
  where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test; by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run;

/*** RLS ***/
proc sql;
  create table test as
  select nonapp2.pcode, nonapp2.munic, numero, nba, pccfdups.propmunic_RLS, pccfdups.diffmatmunic, pccfdups.diffsocmunic, 
         pccfdups.fortcp, pccfdups.nuniq, pccfdups.fortmunic, pccfdups.zim, pccfdups.zone,pccfdups.centmat, pccfdups.rss, 
         pccfdups.nombre, pccfdups.clsc, pccfdups.rls_code, pccfdups.frac2, pccfdups.quartmatrls, pccfdups.quartsocrls
  from nonapp2, pccfdups
  where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RSS ***/
proc sql;
  create table test as
  select nonapp2.pcode, nonapp2.munic, numero, nba, pccfdups.first, pccfdups.propmunic, pccfdups.diffmatmunic, pccfdups.diffsocmunic, 
         pccfdups.fortcp, pccfdups.nuniq, pccfdups.fortmunic, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.centmat, pccfdups.centsoc,
         pccfdups.zim, pccfdups.zone, pccfdups.rss, pccfdups.nombre, pccfdups.clsc, pccfdups.rls_code, pccfdups.frac,
         pccfdups.quartmatrss, pccfdups.quartsocrss, pccfdups.quintmatRSS, pccfdups.quintsocRSS
  from nonapp2, pccfdups
  where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp2;by numero;run; 

data appmunic3 fichin;
  merge nonapp2(in=in1) test1(in=in2) test2(in=in2a) test3(in=in2b);
  by numero;
  if in1*in2*in2a*in2b then output appmunic3;
  else if in1 then output fichin;
run;

%end;

/*********************************************************
  * Assignation des indices de dfavorisation  partir ***
  * du code postal seulement                           ***
  * Fichier FCCP+                                      ***
  *******************************************************/
/*** Avec WCF *****/
data wcf2;
  set wcf;
run;

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf2.propcp_CLSC, wcf2.diffmatcp, wcf2.diffsoccp, wcf2.fortmunic, 
         wcf2.fortcp, wcf2.centmat, wcf2.zim, wcf2.zone, wcf2.rss, wcf2.nuniq, wcf2.nombre, wcf2.clsc, wcf2.rls_code, 
         wcf2.frac, wcf2.quartmatclsc, wcf2.quartsocclsc
  from fichin, wcf2
  where fichin.pcode=wcf2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf2.propcp_RLS, wcf2.diffmatcp, wcf2.diffsoccp, wcf2.fortmunic, 
         wcf2.fortcp, wcf2.centmat, wcf2.zim, wcf2.zone, wcf2.rss, wcf2.nuniq, wcf2.nombre, wcf2.clsc, wcf2.rls_code, 
         wcf2.frac, wcf2.quartmatrls, wcf2.quartsocrls
  from fichin, wcf2
  where fichin.pcode=wcf2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;


/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				  else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, nba, wcf2.first, wcf2.propcp, wcf2.diffmatcp, wcf2.diffsoccp, wcf2.fortmunic, 
         wcf2.fortcp, wcf2.nuniq, wcf2.quintmat, wcf2.quintsoc, wcf2.centmat, wcf2.centsoc, wcf2.zim, wcf2.zone, 
         wcf2.rss, wcf2.nombre, wcf2.clsc, wcf2.quartmatrss, wcf2.quartsocrss, wcf2.quintmatRSS, wcf2.quintsocRSS
  from fichin, wcf2
  where fichin.pcode=wcf2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=fichin;by numero;run; 

data appcp1 nonapp3 ;
  merge fichin(in=in1) test1(in=in2) test2(in=in2a) test3(in=in2b);
  by numero;
  if in1*in2*in2a*in2b then output appcp1;
  else if in1 then output nonapp3;
run;

/*** Avec PCCFUINIQ *****/
proc sort data=nonapp3;by pcode;run;
proc sort data=pccfuniq;by pcode;run;

/*** CLSC ***/
proc sql;
  create table test as
  select nonapp3.pcode, nonapp3.numero, nonapp3.nba, pccfuniq.propcp_CLSC, pccfuniq.propmunic_CLSC, pccfuniq.rss, pccfuniq.nombre, 
         pccfuniq.clsc, pccfuniq.rls_code, pccfuniq.frac,pccfuniq.centmat,pccfuniq.nuniq, pccfuniq.quartmatclsc, pccfuniq.quartsocclsc
  from nonapp3, pccfuniq
  where nonapp3.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par CLSC ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1;
      output; 
    end;
  end;
run; 

/*** RLS ***/
proc sql;
  create table test as
  select nonapp3.pcode, nonapp3.numero, nonapp3.nba, pccfuniq.propcp_RLS, pccfuniq.propmunic_RLS, pccfuniq.rss, pccfuniq.nombre, 
         pccfuniq.clsc, pccfuniq.rls_code, pccfuniq.frac,pccfuniq.centmat,pccfuniq.nuniq, pccfuniq.quartmatrls, pccfuniq.quartsocrls
  from nonapp3, pccfuniq
  where nonapp3.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par RLS ***/

data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RSS ***/
proc sql;
  create table test as
  select nonapp3.pcode, nonapp3.numero, nba, pccfuniq.propcp, pccfuniq.diffmatcp, pccfuniq.diffsoccp, pccfuniq.fortmunic, 
         pccfuniq.fortcp, pccfuniq.nuniq, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc,
	     pccfuniq.zim, pccfuniq.zone, pccfuniq.rss, pccfuniq.nombre, pccfuniq.clsc, pccfuniq.rls_code, pccfuniq.frac,
	     pccfuniq.quartmatrss, pccfuniq.quartsocrss, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS
  from nonapp3, pccfuniq
  where nonapp3.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp3;by numero;run; 

data appcp2 nonapp4 ;
  merge nonapp3(in=in1) test1(in=in2) test2(in=in2a) test3(in=in2b);
  by numero;
  if in1*in2*in2a*in2b then output appcp2;
  else if in1 then output nonapp4;
run;

/*** Avec PCCFDUPS *****/
data pccfdup2;
  set pccfdups;
run;

/*** CLSC ***/
proc sql;
  create table test as
  select nonapp4.pcode, nonapp4.numero, nonapp4.nba, pccfdup2.propcp_CLSC, pccfdup2.diffmatcp, pccfdup2.diffsoccp, pccfdup2.fortcp, 
         pccfdup2.fortmunic,pccfdup2.nuniq, pccfdup2.zim, pccfdup2.zone, pccfdup2.rss, pccfdup2.centmat, pccfdup2.nombre, 
         pccfdup2.clsc, pccfdup2.rls_code, pccfdup2.frac, pccfdup2.quartmatclsc, pccfdup2.quartsocclsc
  from nonapp4, pccfdup2
  where nonapp4.pcode=pccfdup2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RLS ***/
proc sql;
  create table test as
  select nonapp4.pcode, nonapp4.numero, nonapp4.nba, pccfdup2.propcp_RLS, pccfdup2.diffmatcp, pccfdup2.diffsoccp, pccfdup2.fortcp, 
         pccfdup2.fortmunic,pccfdup2.centmat, pccfdup2.zim, pccfdup2.zone, pccfdup2.rss, pccfdup2.nuniq, pccfdup2.nombre, 
         pccfdup2.clsc, pccfdup2.rls_code, pccfdup2.frac, pccfdup2.quartmatrls, pccfdup2.quartsocrls
     from nonapp4, pccfdup2
     where nonapp4.pcode=pccfdup2.pcode
	 order by numero,nuniq;
    quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RSS ***/

proc sql;
  create table test as
  select nonapp4.pcode, nonapp4.numero, nba, pccfdup2.first, pccfdup2.propcp, pccfdup2.diffmatcp, pccfdup2.diffsoccp, 
         pccfdup2.fortmunic, pccfdup2.fortcp,pccfdup2.nuniq, pccfdup2.quintmat, pccfdup2.quintsoc, pccfdup2.centmat, pccfdup2.centsoc,
		 pccfdup2.zim, pccfdup2.zone, pccfdup2.rss, pccfdup2.nombre, pccfdup2.clsc, pccfdup2.rls_code, pccfdup2.frac,
		 pccfdup2.quartmatrss, pccfdup2.quartsocrss, pccfdup2.quintmatRSS, pccfdup2.quintsocRSS
  from nonapp4, pccfdup2
  where nonapp4.pcode=pccfdup2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
	cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp4;by numero;run; 

data appcp3 nonapp5;
  merge nonapp4(in=in1) test1(in=in2) test2(in=in2a) test3(in=in2b);
  by numero;
  if in1*in2*in2a*in2b then output appcp3;
  else if in1 then output nonapp5;
run;

data tout;
  set %if &munic ne 0 %then %do; appmunic1(in=in1) appmunic2(in=in2) appmunic3(in=in3) %end;
      appcp1(in=in4) appcp2(in=in5) appcp3(in=in6) nonapp5(in=in7);
  if in7 then direct=0;
  if in5 then do; 
    direct=1;
    diffmatcp=0; 
    diffsoccp=0;
  end;
  %if &munic ne 0 %then %do; 
    if in1 then base=1;
    else if in2 then do; 
      direct=1; 
      diffmatmunic=0; 
      diffsocmunic=0; base=2;
	end;
	else if in3 then base=3;
	diffmat=diffmatmunic;
	diffsoc=diffsocmunic;
  %end;
	if in4 then base=1;
	else if in5 then base=2;
	else if in6 then base=3;
	if diffmatcp ne . then diffmat=diffmatcp;
 	if diffsoccp ne . then diffsoc=diffsoccp;

	if in7=1 then do;
      quintmat=0;quintsoc=0;
      quartmatclsc=0;quartsocclsc=0;
      quartmatrls=0;quartsocrls=0;
      quartmatrss=0;quartsocrss=0;
      quintmatRSS=0;quintsocRSS=0;
    end;
	if quintmat=0 then groupe=6;
	else if quintmat=. then groupe=5;
	else if direct=1 then groupe=1;
    else if diffmat=0 and diffsoc=0 then groupe=2;
	else if fortcp=1 or fortmunic=1 then groupe=3;
    else groupe=4;

    if propmunic_CLSC ne . then propCLSC=propmunic_CLSC;
	else if propcp_CLSC ne . then propCLSC=propcp_CLSC;
	else propCLSC=100;

	if propmunic_RLS ne . then propRLS=propmunic_RLS;
	else if propcp_RLS ne . then propRLS=propcp_RLS;
	else propRLS=100;

	if propmunic ne . then prop=propmunic;
	else if propcp ne . then prop=propcp;
	else prop=100;
	css=zim;
  keep numero base groupe CentMat CentSoc QuintMat QuintSoc QuintMatRSS QuintSocRSS QuartMatRSS QuartSocRSS QuartMatRLS QuartSocRLS QuartMatCLSC quartsocCLSC RSS zone CSS;
run;

proc sort; by numero; run; 

data fichin;
  set &in;
  numero=_n_;
run;

data tout;
  retain CentMat CentSoc QuintMat QuintSoc QuintMatRSS QuintSocRSS QuartMatRSS QuartSocRSS QuartMatRLS QuartSocRLS QuartMatCLSC quartsocCLSC RSS zone CSS base groupe numero;
  set tout;
run;

data &out;
  merge fichin tout;
  by numero;
  DATE = '20202204';
  format groupe fgroupe. base fbase.;
  drop numero;
run;

proc datasets library=work;
  delete corr2 wcf2 pccfuniq2 pccfdups2 test test1 test2 test3 fichin tout appmunic1 appmunic2 appmunic3 appmunic4 appcp1 
         appcp2 appcp3 appcp4 nonapp1 nonapp2 nonapp3 nonapp4 nonapp5 nonapp6 nonapp7 pccfdup2; 
quit;
%mend Assignation01;

proc format;
  value fgroupe 1="Assignation CP - Indice unique"
  		        2="Assignation non unique mais tous les AD mnent au mme indice"
				3="Assignation non unique mais avec prdominance d'un AD  plus de 90%"
				4="Autre assignation non unique"
				5="Appartient  un AD sans indice"
				6="Aucun appariement CP - AD - Indice";
  value fbase 1="WCF"
			  2="PCFF unique"
			  3="PCCF dups";
 run;

/****************************************************************
* Exemple: %assignation(dec04, cp, cod_mun, dec04v2); 	    	*
* - Le fichier en entre est WORK.dec04              			*
* - La variable pour le code postal est CP             			*
* - La variable pour le code municipal est COD_MUN     			*
*   N.B.: on doit mettre 0 (zro) si on ne veut pas faire   	*
*   l'assignation  l'aide du code municipal     				*
* - Le fichier rsultant est WORK.dec04v2. Il est identique 	*
*   au fichier d'entre, mais avec l'indice de dfavorisation  	*
*   en quintiles, en quartiles, la rgion et la variable zone  	*
* - NOTE: Les rsultats peuvent varier si l'ordre du fichier  	*
*   d'entre n'est pas la mme.                					*
****************************************************************/ 

%Assignation01(in=i2001.dec04,pcode=cp,munic=cod_mun,out=i2001.dec04v2);
